@@ -44,6 +44,7 @@
 #include <KPasswordDialog>
 #include <KShell>
 #include <KApplication>
+#include <KWindowSystem>
 
 #include "kdesudo.h"
 
@@ -71,10 +72,18 @@ KdeSudo::KdeSudo(const QString& icon, const QString& appname)
 	emptyPwd = args->isSet("s");
 	QString runas = args->getOption("u");
 	QString cmd;
+	int winid = -1;                           
+	bool attach = args->isSet("attach");
 
 	m_dialog = new KPasswordDialog(0);
 	m_dialog->setDefaultButton( KDialog::Ok );
 
+	if (attach)                              
+	{                                         
+		winid = args->getOption("attach").toInt(&attach, 0);
+		KWindowSystem::setMainWindow(m_dialog, (WId)winid);
+	}
+
 	if (!args->isSet("c") && !args->count() && (!args->isSet("s")))
 	{
 		KMessageBox::information(0, i18n("No command arguments supplied!\n"
@@ -187,6 +196,7 @@ KdeSudo::KdeSudo(const QString& icon, const QString& appname)
 
 			if (!runas.isEmpty())
 				*p << "-u" << runas;
+			*p << "--";
 		}
 
 		if (realtime)
@@ -194,6 +204,7 @@ KdeSudo::KdeSudo(const QString& icon, const QString& appname)
 			*p << "nice" << "-n" << "10";
 			m_dialog->addCommentLine(i18n("Priority:"), i18n("realtime:") + 
 				QChar(' ') + QString("50/100"));
+			*p << "--";
 		}
 		else if (priority)
 		{
@@ -206,22 +217,18 @@ KdeSudo::KdeSudo(const QString& icon, const QString& appname)
 
 			*p << "nice" << "-n" << strn;
 			m_dialog->addCommentLine(i18n("Priority:"), n + QString("/100"));
+			*p << "--";
 		}
 
-		*p << "--";
+
 
 		if (args->isSet("c"))
 		{
 			QString command = args->getOption("c");
-			QStringList commandSplit = KShell::splitArgs(command);
-			for (int i = 0; i < commandSplit.count(); i++)
-			{
-				*p << validArg(commandSplit[i]);
-				if (i == 0)
-					cmd += validArg(commandSplit[i]) + QChar(' ');
-				else
-					cmd += KShell::quoteArg(validArg(commandSplit[i])) + QChar(' ');
-			}
+			cmd += command;
+			*p << "sh";
+			*p << "-c";
+			*p << command;
 		}
 
 		else if (args->count())
@@ -299,8 +306,13 @@ void KdeSudo::parseOutput()
 	{
 		badpass++;
 		if (badpass == 1)
+		{
 			m_dialog->addCommentLine(i18n("<b>Warning: </b>"), i18n("<b>Incorrect password, please try again.</b>"));
-		else if (badpass > 2)
+			m_dialog->show();
+		}
+		else if (badpass == 2)
+			m_dialog->show();
+		else
 			error(i18n("Wrong password! Exiting..."));
 		
 	}
